/*
 *   1997-2000 Metrowerks Corp.
 *
 *  Questions and comments to:
 *       <mailto:support@metrowerks.com>
 *       <http://www.metrowerks.com/>
 */

#include <Fonts.h>
#include <Menus.h>
#include <MacWindows.h>
#include <Dialogs.h>
#include <QuickDraw.h>
#include <TextEdit.h>

#include "Crrrhaaack.h"
#include "EnergyStar.h"

#define	MY_ABS(x)			((x) >= 0 ? (x) : -(x))

/* --------------------------------------------------------------------------------	*/
static void Initialize(void)
{
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
}

/* --------------------------------------------------------------------------------	*/
static pascal void DeviceLoopProc(short depth, short, GDHandle device, long)
{
	SetCurrentA5();
	SetDepth(device, 8, 1, 1);
	SetDepth(device, depth, 1, 1);
}

/* --------------------------------------------------------------------------------	*/
static void Refresh(void)
{
	DeviceLoop(NULL, DeviceLoopProc, 0L, allDevices);
}

/* --------------------------------------------------------------------------------	*/
static void GetScreenSize(Rect *rect)
{
	CrunchData	*data;
	
	if(Gestalt(kCrunchCreator, (long *)&data) == noErr)
		*rect = data->screenRect;
}

/* --------------------------------------------------------------------------------	*/
static void SetScreenSize(Rect *rect)
{
	CrunchData	*data;
	
	if(Gestalt(kCrunchCreator, (long *)&data) == noErr)
	{
		data->screenRect = *rect;
		Refresh();	
	}
}

//---------------------------------------------------------------------------------
static void MakeSmallRect(Rect *screenRect, Rect *itemRect, Rect *smallRect)
{
	*smallRect = *screenRect;
	OffsetRect(smallRect, -smallRect->left, -smallRect->top);
	smallRect->right = (smallRect->right + 4) / 5;
	smallRect->bottom = (smallRect->bottom + 4) / 5;
	OffsetRect(smallRect, screenRect->left / 5 + itemRect->left, 
			   screenRect->top / 5 + itemRect->top);
}

//---------------------------------------------------------------------------------
static void MakeScreenRect(Rect *smallRect, Rect *itemRect, Rect *screenRect)
{
	*screenRect = *smallRect;
	OffsetRect(screenRect, -screenRect->left, -screenRect->top);
	screenRect->right = screenRect->right * 5;
	screenRect->bottom = screenRect->bottom * 5;
	OffsetRect(screenRect, (smallRect->left - itemRect->left) * 5, 
			   (smallRect->top - itemRect->top) * 5);
}

//---------------------------------------------------------------------------------
static pascal Boolean DialogFilter(DialogPtr theDlog, EventRecord *theEvent, short *itemHit)
{
	Rect 	*bigRect 	= (Rect *)GetWRefCon(theDlog);
	short	iType;
	Handle	iHandle;
	Rect	iRect, r, oldRect;
	Point	curMousePt, whichCorner;
	Boolean	changed;
	GrafPtr	savedPort;
	
	GetPort(&savedPort);
	SetPort(theDlog);
	GetDialogItem(theDlog, 3, &iType, &iHandle, &iRect);
	GetMouse(&curMousePt);						/* Get current mouse pos		*/

	if(theEvent->what == mouseDown && PtInRect(curMousePt, &iRect))
	{
		MakeSmallRect(bigRect, &iRect, &r);
		whichCorner.h = (MY_ABS(curMousePt.h - r.left) > MY_ABS(curMousePt.h - r.right));
		whichCorner.v = (MY_ABS(curMousePt.v - r.top) > MY_ABS(curMousePt.v - r.bottom));
		
		PenSize(2, 2);
		PenMode(srcXor);
		PenPat(&qd.gray);

		while(Button())
		{
			oldRect = r;
			changed = false;
			GetMouse(&curMousePt);						/* Get current mouse pos		*/

			if(whichCorner.h && curMousePt.h != r.right)
			{
				changed = true;
				r.right = curMousePt.h;
			}
			else if(!whichCorner.h && curMousePt.h != r.left)
			{
				changed = true;
				r.left = curMousePt.h;
			}
			
			if(whichCorner.v && curMousePt.v != r.bottom)
			{
				changed = true;
				r.bottom = curMousePt.v;
			}
			else if(!whichCorner.v && curMousePt.v != r.top)
			{
				changed = true;
				r.top = curMousePt.v;
			}
			
			if(changed)
			{
				FrameRect(&oldRect);			/* Erase old					*/
				FrameRect(&r);					/* and draw new.				*/
			}
		}
		MakeScreenRect(&r, &iRect, bigRect);
	}
	SetPort(savedPort);
	return StdFilterProc(theDlog, theEvent, itemHit);
}

/* --------------------------------------------------------------------------------	*/
pascal void DrawScreen(DialogRef theDlog, short)
{
	Rect 		*bigRect 	= (Rect *)GetWRefCon(theDlog);
	short		iType;
	Handle		iHandle;
	Rect		iRect, r;
	PicHandle	thePict = GetPicture(1);
	
	PenNormal();
	GetDialogItem(theDlog, 3, &iType, &iHandle, &iRect);
	DrawPicture(thePict, &iRect);
	PenSize(2, 2);
	PenMode(srcXor);
	PenPat(&qd.gray);
	MakeSmallRect(bigRect, &iRect, &r);
	FrameRect(&r);
}

/* --------------------------------------------------------------------------------	*/
void main(void)
{
	Rect			r;
	DialogRef		theDlog;
	short			hit = 0;
	short			iType;
	Handle			iHandle;
	Rect			iRect;
	long			oldA5, portAddress;

	Initialize();
	if((theDlog = GetNewDialog(128, NULL, (WindowRef)-1)) != NULL)
	{
		GetScreenSize(&r);
		SetWRefCon(theDlog, (long)&r);
		GetDialogItem(theDlog, 3, &iType, &iHandle, &iRect);
		SetDialogItem(theDlog, 3, iType, (Handle)DrawScreen, &iRect);
		SetDialogDefaultItem(theDlog, 1);
		ShowWindow(theDlog);
		while(hit != 1)
		{
			ModalDialog(DialogFilter, &hit);
			if(hit == 1 || hit == 2)
			{
				if(r.left < 0) r.left = 0;
				if(r.top < 0) r.top = 0;
				if(r.right > 1024) r.right = 1024;
				if(r.bottom > 768) r.right = 768;
				SetScreenSize(&r);
			}
		}
		DisposeDialog(theDlog);
	}
}