//
//  Archive.java
//  Mentat
//
//	API:
//
//		(all accessors for [textResponse, key, value, arch])
//		predicates: keyPresent(String), errorText()
//
//		methods: String lookupValue(String), addEntry(String, String), removeEntry(String),
//					returnToDefaults(), saveArch(ObjectOutputStream), loadArch(ObjectInputStream)
//
//  Created by sziegler on Wed Jun 20 2001.
//  Copyright (c) 2001 Wormsign Code Implementations. All rights reserved fnord.
/*
This file and its intellectual contents are considered property of the creator and are to be treated as such with respect to use in other applications.  Any use of this software for any purpose whatsoever must have explicit permission from the author of the file.  This code is part of an ongoing development process for future possible products, and so is considered private property.
Do not use without permission.  Author: Scott C. Ziegler <Aslan@Narnia.net>
*/

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.awt.*;
import java.io.*;
import java.util.*;

public class Archive {
    
    public static final String DEFAULT_KEY = "fnord";
    public static final String DEFAULT_VALUE = "a small powerful thing.";
    public static final String NOT_FOUND = "<Entry Not Found!>";
    
    // Data Members
    
    String name;
    
    boolean	textResponse;
    String	keyStr;
    String	valueStr;
    Hashtable	arch;
    
    // Constructors
    
    public Archive() {
    	name = new String("");
        textResponse = true;
        keyStr = new String(DEFAULT_KEY);
        valueStr = new String(DEFAULT_VALUE);
        arch = new Hashtable(100);
        }
        
    // Selectors
    
    public String getName() { return name; } 
    public boolean getTextResponse() { return textResponse; }
    public String getKey() { return keyStr; }
    public String getValue() { return valueStr; }
    public Hashtable getArch() { return arch; }
    
    // Mutators
    
    public void setName(String nm) { name = nm; }
    public void setTextResponse(boolean response) { textResponse = response; }
    public void setKey(String newKey) { keyStr = newKey; }
    public void setValue(String newValue) { valueStr = newValue; }
    public void setArch(Hashtable newArch) { arch = newArch; }
    
    // Predicates
    
    public boolean keyPresent(String key) { return arch.containsKey(key); }
    public boolean errorText() { return textResponse; }
    
    // Member Functions
    
    public String lookupValue(String key) { 
        valueStr = (String)arch.get(key);
        if (valueStr != null) return valueStr;
        else if (textResponse) { return NOT_FOUND; }
             else { return null; }
        }
        
    public void addEntry(String key, String value) { arch.put(key, value); }
        
    public void removeEntry(String key) { arch.remove(key); }
    
    public void returnToDefaults() {
        keyStr = DEFAULT_KEY;
        valueStr = DEFAULT_VALUE;
        arch.clear();
        }
        
    public void saveArch(ObjectOutputStream oos) {
        try {
            oos.writeObject(arch);
            }
        catch (IOException ioe) { 
            if (textResponse) { System.out.println("Error: " + ioe.toString()); } 
            }
        }
        
    public void loadArch(ObjectInputStream ois) {
        try {
            arch = (Hashtable)ois.readObject();
            }
        catch (IOException ioe) { 
            if (textResponse) { System.out.println("Error: " + ioe.toString());}
            }
        catch (ClassNotFoundException cnfe) { 
            if (textResponse) { System.out.println("Error: " + cnfe.toString());}
            }
        }
    }
