// events.cpp

#include "events.h"
#include "drawing.h"
#include "Finder_res.h"

// timing constants
const int beep_ms = 250;
const int after_beep_ms = 500;
const int happy_ms = 3000;
const int welcome_ms = 3000;


//
// send_menu_event()
//
void send_menu_event(int id, Boolean pen_down, int x, int y) {
	EventType event;
	
	event.eType = 		menuEvent;
	event.penDown = 	pen_down;
	event.screenX =		x;
	event.screenY =		y;
	event.data.menu.itemID =	id;
	
	EvtAddEventToQueue (&event);
}

//
// send_update_event()
//
void send_update_event() {
	int	id = FrmGetActiveFormID();
	
	FrmUpdateForm (id, frmRedrawUpdateCode);
}

//
// sys_beep()
//
void sys_beep(int duration) { // in milliseconds
	// constants
	const UInt32			note_frequency = 880; // hertz
	
	// variables
	UInt32					volume;
	Err						err;
	SndCommandType	cmd;
	
	// instructions
	volume = PrefGetPreference(prefSysSoundVolume);
	cmd.cmd = sndCmdFrqOn;
	cmd.param1 = note_frequency; // hertz
	cmd.param2 = duration; // milliseconds
	cmd.param3 = volume;
	err = SndDoCmd(NULL, &cmd, 0);
}


//
// delay()
//
void delay(int duration) { // in milliseconds
	Int32	ticks_per_sec = SysTicksPerSecond();
	Int32	total_ticks = Int32 ( ticks_per_sec * Int32(duration) / 1000);
	Err		err;
	
	err = SysTaskDelay (total_ticks);
}


//
// startup_sequence()
//
void startup_sequence() {
	// variables
    FormPtr frmP;

	// show blank startup screen
	frmP = FrmInitForm(StartupForm);
	if (frmP==NULL) return;
	FrmDrawForm(frmP);
	// beep for one quarter second
	sys_beep(beep_ms);
	delay(after_beep_ms);
	// show happy mac for one second
	draw_bitmap(StartupHappyMacBitmap, 0, 0, left_align, top_align);
	delay(happy_ms);
	// show welcome msg for five seconds
	draw_bitmap(StartupWelcometoMacBitmap, 0, 0, left_align, top_align);
	delay(welcome_ms);
	// delete form
	FrmDeleteForm(frmP);
	
}


