/*
	File:		MoreOSUtils.h

	Contains:	Interface to the OS utilities library.

	Written by:	Quinn

	Copyright:	Copyright  1999-2000 by Apple Computer, Inc., all rights reserved.

	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Change History (most recent first):

         <5>    24/11/00    Quinn   Make interrupt mask handling stuff CALL_NOT_IN_CARBON.
         <4>    24/11/00    Quinn   Correct routine name in previous checkin comment.
         <3>    24/11/00    Quinn   Carbon does not support 68K code generation, so I've made
                                    MakeData68KExecutable CALL_NOT_IN_CARBON. I could have made
                                    this work, at least on Mac OS 9, but I don't see the need right
                                    now.
         <2>     16/3/99    Quinn   Rolled in InterfaceDisableLib from DTS Technote 1137.
         <1>      1/3/99    Quinn   First checked in.
*/

#pragma once

/////////////////////////////////////////////////////////////////

// MoreIsBetter Setup

#include "MoreSetup.h"

// Standard Mac OS interfaces

#include <MacTypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/////////////////////////////////////////////////////////////////
// Instruction Cache Flushing

extern pascal OSStatus MakeDataPowerPCExecutable(void *address, ByteCount count);
	// Make the memory range described by address and count
	// executable as PowerPC code.  This routine can only return
	// an error when called from classic 68K code.

#if CALL_NOT_IN_CARBON

extern pascal OSStatus MakeData68KExecutable(void *address, ByteCount count);
	// Make the memory range described by address and count
	// executable as 68K code.  This routine can never return an
	// error.  It has an error result purely for symmetry with
	// MakeDataPowerPCExecutable.

#endif

#if TARGET_CPU_68K && !TARGET_RT_MAC_CFM

	// IMPORTANT:
	// This routine is only necessary if you're compiling
	// classic 68K which calls MakeDataPowerPCExecutable.

	extern pascal void TermMoreOSUtils(void);
		// A termination routine for this library.  Call it before
		// you unload from memory.

#endif

/////////////////////////////////////////////////////////////////
// Interrupt Enable and Disable

// See DTS Technote 1xxx "Disabling Interrupts on the Traditional Mac OS"
// for details on how this works, why you should avoid it, and what
// alternatives there are.

#if CALL_NOT_IN_CARBON

extern pascal UInt16 GetInterruptMask(void);
	// Returns the current interrupt mask, as a value
	// from 0 to 7.
	
extern pascal UInt16 SetInterruptMask(UInt16 newMask);
	// Sets the current interrupt mask, as a value
	// from 0 to 7.

#endif

#ifdef __cplusplus
}
#endif
