/*
	SpellCompositor.java
		- development pre-cursor to ArcanaApp, used for experimentation and debugging/testing
	
	Written by Scott C. Ziegler for use with Simulcra RPG and AD&D
	
	vers. history:
		12.21.00 - added comment section, fixed navigation buttons, added book info accessors.
	
	Work Needed:
		- need to implement navigation buttons, make sure they don't move unless current spell is saved!
		- need to check for duplicates before adding a spell.
		- fix up removal code with JDialog/JOptionPane.
	Work Completed:
		Date - Desc


*/

/*
This file and its intellectual contents are considered property of the creator and are to be treated as such with respect to use in other applications.  Any use of this software for any purpose whatsoever must have explicit permission from the author of the file.  This code is part of an ongoing development process for future possible products, and so is considered private property.
Do not use without permission.  Author: Scott C. Ziegler <Aslan@Narnia.net>
*/

//package Arcana;

// initEditor needs work (see comments)

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;


public class SpellCompositor {
	
	static final String DEFAULT_SPELLFILE_PATH = "untitled.spellbook";
	
	InteractionFrame		hostFrame;
	Thoth.SpellEditFrame	editFrame;
	Thoth.SpellViewFrame	viewFrame;
	Thoth.SpellBrowserFrame	browserFrame;
	
	Arcana.SpellFile	spellFile;
	Arcana.SpellBook	spellBook;
	
	Arcana.SpellRecord	lastSpell;
	Arcana.SpellRecord	currentSpell;
	
	int			currentLevel;
	int			currentIndexInLevel;
	boolean		fileTouched;
	
	FileReader 		fr;
	BufferedReader 	in;
	
	char[] 	buffer;
	int 	bufferIndex = 0;
	int 	bytesRead;
	short 	asciiChar;
	
	public SpellCompositor() {
		// IMPL. Vars.
		fileTouched = false;
		currentLevel = 1;
		currentIndexInLevel = 0;
		// GUI
		hostFrame = new InteractionFrame();
		editFrame = new Thoth.SpellEditFrame();
		initEditor();
		viewFrame = new Thoth.SpellViewFrame();
		initViewer();
		browserFrame = new Thoth.SpellBrowserFrame();
		initBrowser();
		// MODEL
		spellFile = new Arcana.SpellFile(DEFAULT_SPELLFILE_PATH);
		spellBook = new Arcana.SpellBook();
		lastSpell = new Arcana.SpellRecord();
		currentSpell = new Arcana.SpellRecord();
		}
	
	public SpellCompositor(InteractionFrame hFrame) {
		// IMPL. Vars.
		fileTouched = false;
		currentLevel = 1;
		currentIndexInLevel = 0;
		// GUI
		hostFrame = hFrame;
		editFrame = new Thoth.SpellEditFrame();
		initEditor();
		viewFrame = new Thoth.SpellViewFrame();
		initViewer();
		browserFrame = new Thoth.SpellBrowserFrame();
		initBrowser();
		// MODEL
		spellFile = new Arcana.SpellFile(DEFAULT_SPELLFILE_PATH);
		spellBook = new Arcana.SpellBook();
		lastSpell = new Arcana.SpellRecord();
		currentSpell = new Arcana.SpellRecord();
		}
		
	public void mainline() {
		String selector;
		
		while(true) {
			selector = hostFrame.getLine();
			
				 if (selector.compareTo("") == 0) {
					hostFrame.printEndLine("You didn't enter anything!");
					}
			else if (selector.compareTo("test?") == 0) {
					hostFrame.printEndLine("SpellCompositor mainline");
					}
			else if (selector.compareTo("Commands?") == 0 ||
					 selector.compareTo("?") == 0) {
					 hostFrame.printEndLine("?, about, "
					 						+ "createSpell, "
					 						+ "parseSpell, "
					 						+ "removeSpell, "
					 						+ "save, "
					 						+ "load, "
					 						+ "displaySpell, "
					 						+ "findSpell, "
					 						+ "viewBook, "
					 						+ "noOfSpells, "
					 						+ "showBookInfo, "
					 						+ "registerBook, "
					 						+ "Quit");
					}
			else if (selector.compareTo("about") == 0) {
					about();
					}
			else if (selector.compareTo("createSpell") == 0) {
					createSpell();
					}
			else if (selector.compareTo("parseSpell") == 0) {
					parseSpell();
					}
			else if (selector.compareTo("removeSpell") == 0) {
					removeSpell();
					}
			else if (selector.compareTo("save") == 0) {
					save();
					}
			else if (selector.compareTo("load") == 0) {
					load();
					}
			else if (selector.compareTo("displaySpell") == 0) {
					displaySpell();
					}
			else if (selector.compareTo("findSpell") == 0) {
					findSpellInBook();
					}
			else if (selector.compareTo("viewBook") == 0) {
					showBrowser();
					//viewBook();
					}
			else if (selector.compareTo("noOfSpells") == 0) {
					hostFrame.printEndLine("# of Spells: " + spellBook.getNoOfSpellsInBook());
					}
			else if (selector.compareTo("showBookInfo") == 0) {
					hostFrame.printEndLine("Spellbook " + spellBook.getName() 
							+ ", created by " + spellBook.getCreator() 
							+ " and owned by " + spellBook.getOwner() + ".");
					}
			else if (selector.compareTo("registerBook") == 0) {
					registerBook();
					}
			else if (selector.compareTo("fuckTouch") == 0) {
					fileTouched = false;
					hostFrame.printEndLine("Touch is fucked.  Go ahead and load!");
					}
			else if (selector.compareTo("Quit") == 0) {	
					hostFrame.printLine("Exiting Shell... Quiessence.");
					System.exit(0);
					break;
					}
			else { // default case
				hostFrame.printEndLine("The command \""+ selector +"\" was not recognized!");
				}
			}
		}
		
	public void about() {
		hostFrame.printLine("Spell Compositor - Arcana SpellBook");
		hostFrame.printLine("written by Scott C. Ziegler, 2000-2001");
		hostFrame.printLine("presented at MacHack 2001 on MacOS X");
		hostFrame.printLine("     This small application was developed during experimentations with the Java programming language and its Java Foundation Classes, including Swing.  Meant as an \"in-house\" editing program, this app flourished into an ALMOST useful spellbook archiving program.  This app is dedicated to my friends from days of old when magick filled the aire, and to those who have ever felt the age-smoothed edges of a polyhedron as it decided the fate of some hapless character.");
		hostFrame.printEndLine("You never saw this fnord.");
		}
		
		
	private void createSpell() {
		showEditor();
		editFrame.clearFields();
		}
		
	private void parseSpell() {
		buffer = new char[4096];
		bufferIndex = 0;
		bytesRead = 0;
		Arcana.SpellRecord spellRec = new Arcana.SpellRecord();
		String selector = new String("");
		
		while(true) {
			hostFrame.printEndLine("Enter filename of text file to parse as a spell:");
			selector = hostFrame.getLine();
			
			if (selector.compareTo("") == 0) {
				hostFrame.printEndLine("That is an invalid filename!");
				continue;
				}
			else {
				try {
					fr = new FileReader(selector);
					in = new BufferedReader(fr);
						
					spellRec.setLevel(Integer.parseInt(in.readLine()));
					spellRec.setName(in.readLine());
					hostFrame.printLine("Name: " + spellRec.getName());
					spellRec.setSchool(in.readLine());
					//hostFrame.printLine("School: " + spellRec.getSchool());
					spellRec.setRange(in.readLine());
					spellRec.setComponentsByString(in.readLine());
					spellRec.setDuration(in.readLine());
					spellRec.setCastingTime(in.readLine());
					spellRec.setAreaOfEffect(in.readLine());
					spellRec.setSavingThrow(in.readLine());
					while((bytesRead = in.read()) != -1) {
						asciiChar = (short)bytesRead;
						buffer[bufferIndex] = (char)asciiChar;
						bufferIndex++;
						}
					spellRec.setEffect(new Arcana.SpellDescription(new String(buffer,0,bufferIndex)));
					
					hostFrame.printLine("File parsed...");
					in.close();
					lastSpell = spellRec;
					spellBook.addSpellRecord(spellRec);
					touch();
					if (spellBook.getSpellRecordVector(spellRec.getLevel()).contains(spellRec)) {
						hostFrame.printEndLine("Spell Added to Book...");
						}
					else hostFrame.printEndLine("Spell Not Added!  FUCK! A BUG!");
					break;
					}
				catch (FileNotFoundException fnfe) {
					hostFrame.printLine("Error -- " + fnfe.toString());
					hostFrame.printEndLine("Cannot Find the file you requested");
					continue;
					}
				catch (IOException ie) {
					hostFrame.printEndLine("Error -- " + ie.toString());
					break;
					}
				catch (NumberFormatException nfe) {
					hostFrame.printEndLine("Error -- " + nfe.toString());
					//continue;
					break;
					}
				}
			}
		}
		
	private void removeSpell() {
		// Need to add an are you sure dialog...
		spellBook.removeSpellRecord(currentSpell);
		// need to check for if the removal is the last spell in the level
		// need to evaluate this in general... what happens when you delete the last spell?
		if (spellBook.getSpellRecordVector(currentLevel).isEmpty()) currentSpell = new Arcana.SpellRecord();
		else if (!(currentIndexInLevel <= 0)) {
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(currentIndexInLevel-1)
									.getSourceSpell();
			}
		else currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(currentIndexInLevel+1)
									.getSourceSpell();
		}
		
	private void save() {
		if (fileTouched) {
			spellFile.save(spellBook);
			hostFrame.printLine("Saved");
			hostFrame.printEndLine("No. of Spells in Book:" + spellBook.getNoOfSpellsInBook());
			}
		fileTouched = false;
		
		}
		
	private void load() {
		if (!fileTouched) {
			spellBook = spellFile.load();
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(currentIndexInLevel)
									.getSourceSpell();
			hostFrame.printLine("Loaded.");
			hostFrame.printEndLine("No. of Spells in Book:" + spellBook.getNoOfSpellsInBook());
			}
		else hostFrame.printEndLine("Cannot load till changes are saved!");
		}
		
	private void displaySpell() {
		if (lastSpell != null && currentSpell != null) {
			/*
			hostFrame.printLine("Spell Record: ");
			hostFrame.printLine("Level: " + lastSpell.getLevel());
			hostFrame.printLine("Name: " + lastSpell.getName());
			hostFrame.printLine("School: " + lastSpell.getSchool());
			hostFrame.printLine("Range: " + lastSpell.getRange());
			hostFrame.printLine("Components: " + lastSpell.getComponentsString());
			hostFrame.printLine("Duration: " + lastSpell.getDuration());
			hostFrame.printLine("Casting Time: " + lastSpell.getCastingTime());
			hostFrame.printLine("AreaOfEffect: " + lastSpell.getAreaOfEffect());
			hostFrame.printLine("SavingThrow: " + lastSpell.getSavingThrow());
			hostFrame.printEndLine("Description: " + lastSpell.getEffect().getDescription());
			*/
			
			viewFrame.displaySpell(currentSpell);
			showViewer();
			}
		else {
			hostFrame.printEndLine("There isn't a spell yet!");
			}
		}
		
	private void findSpellInBook() {
		String name = new String("");
		String levelStr = new String("");
		int level;
		
		while (true) {
			hostFrame.printEndLine("Enter the name of the spell (exactly).");
			name = hostFrame.getLine();
			hostFrame.printEndLine("Enter the level of the spell or \"!\" if unknown.");
			levelStr = hostFrame.getLine();
			if (levelStr.compareTo("!") == 0) {
				currentSpell = spellBook.findSpellByName(name);
				showViewer();
				break;
				}
			try {
				Integer i = new Integer(levelStr);
				level = i.intValue();
				if (level > 0 && level <= 9) {
					currentSpell = spellBook.findSpell(name, level);
					showViewer();
					}
				break;
				}
			catch (NumberFormatException nfx) {
				hostFrame.printEndLine("Shit Happens when you enter invalid ints.");
				continue;
				}
			}
		}
		
	private void viewBook() {
		String selector;
		Arcana.SpellRecord currentSpell = new Arcana.SpellRecord();
		int currentLevel = 1;
		int currentIndexInLevel = 0;
		
		try {
			hostFrame.printLine("Viewing SpellBook... (help is \"?\")");
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(0)
									.getSourceSpell();
			//System.out.println("current spell initialized");
			while(true) {
				selector = hostFrame.getLine();
				
					 if (selector.compareTo("") == 0) {
						hostFrame.printEndLine("You didn't enter anything!");
						}
				else if (selector.compareTo("test?") == 0) {
						hostFrame.printEndLine("ViewBook");
						}
				else if (selector.compareTo("Commands?") == 0 ||
						 selector.compareTo("?") == 0) {
						 hostFrame.printEndLine("?, displaySpell, "
						 						+ "nextLevel, "
						 						+ "prevLevel, "
						 						+ "levelList, "
						 						+ "nextSpell, "
						 						+ "prevSpell, "
						 						+ "End");
						}
				else if (selector.compareTo("displaySpell") == 0) {
						if (currentSpell != null) {
							hostFrame.printLine("Spell Record: ");
							hostFrame.printLine("Level: " + currentSpell.getLevel());
							hostFrame.printLine("Name: " + currentSpell.getName());
							hostFrame.printLine("School: " + currentSpell.getSchool());
							hostFrame.printLine("Range: " + currentSpell.getRange());
							hostFrame.printLine("Components: " + currentSpell.getComponentsString());
							hostFrame.printLine("Duration: " + currentSpell.getDuration());
							hostFrame.printLine("Casting Time: " + currentSpell.getCastingTime());
							hostFrame.printLine("AreaOfEffect: " + currentSpell.getAreaOfEffect());
							hostFrame.printLine("SavingThrow: " + currentSpell.getSavingThrow());
							hostFrame.printEndLine("Description: " + currentSpell.getEffect().getDescription());
							}
						else {
							hostFrame.printEndLine("There is no spell currently.");
							}
						}
				else if (selector.compareTo("nextLevel") == 0) {
						if (currentLevel + 1 < 9) {
							++currentLevel;
							currentIndexInLevel = 0;
							currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
							hostFrame.printEndLine("Now On Level: " + currentLevel);
							}
						else hostFrame.printEndLine("Maximum Level!");
						}
				else if (selector.compareTo("prevLevel") == 0) {
						if (currentLevel - 1 < 1) {
							--currentLevel;
							currentIndexInLevel = 0;
							currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
							hostFrame.printEndLine("Now On Level: " + currentLevel);
							}
						else hostFrame.printEndLine("Minimum Level!");
						}
				else if (selector.compareTo("levelList") == 0) {
						Vector vec = spellBook.getSpellRegisterVector(currentLevel)
											  .getSpellNamesVector();
						Enumeration e = vec.elements();
						while (e.hasMoreElements()) {
							hostFrame.printLine((String)e.nextElement());
							}
						hostFrame.printEndLine("<End>");
						}
				else if (selector.compareTo("nextSpell") == 0) {
						if (!(currentIndexInLevel > 
								spellBook.getSpellRegisterVector(currentLevel).getSpellRegisterVector().size())) {
							++currentIndexInLevel;
							currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
							hostFrame.printEndLine("Currently At: " + currentSpell.getName());
							}
						else {
							hostFrame.printEndLine("No More Spells!");
							}
						}
				else if (selector.compareTo("prevSpell") == 0) {
						if (!(currentIndexInLevel < 0)) {
							--currentIndexInLevel;
							currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
							hostFrame.printEndLine("Currently At: " + currentSpell.getName());
							}
						else {
							hostFrame.printEndLine("Already at first spell");
							}
						}
				else if (selector.compareTo("End") == 0) {
						hostFrame.printEndLine("Ending ViewBook Routine.");
						break;
						}
				}
			}
		catch(NoSuchElementException nsee) {
			hostFrame.printEndLine("That spell does not exist!");
			}
		}
		
	private void registerBook() {
		String name = new String("");
		String creator = new String("");
		String owner = new String("");
		
		while(true) {
			hostFrame.printEndLine("Enter the Name of the Spellbook:");
			name = hostFrame.getLine();
			
			if (name.compareTo("") == 0) {
				hostFrame.printEndLine("That is an invalid Name!");
				continue;
				}
			else {
				spellBook.setName(name);
				break;
				}
			}
		while(true) {
			hostFrame.printEndLine("Enter the Creator of the Spellbook:");
			creator = hostFrame.getLine();
			
			if (name.compareTo("") == 0) {
				hostFrame.printEndLine("That is an invalid Creator!");
				continue;
				}
			else {
				spellBook.setCreator(creator);
				break;
				}
			}
		while(true) {
			hostFrame.printEndLine("Enter the Owner of the Spellbook:");
			owner = hostFrame.getLine();
			
			if (name.compareTo("") == 0) {
				hostFrame.printEndLine("That is an invalid Owner!");
				continue;
				}
			else {
				spellBook.setOwner(owner);
				break;
				}
			}
		hostFrame.printEndLine("Spellbook " + spellBook.getName() 
							+ ", created by " + spellBook.getCreator() 
							+ " and owned by " + spellBook.getOwner() + ".");
		touch();
		}
		
	private void initEditor() {
		editFrame.setNavigationListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String com = ae.getActionCommand();
					 if (com.compareTo("Back") == 0) {
					 	backSpell();
						}
				else if (com.compareTo("Next") == 0) {
					nextSpell();
					}
				else if (com.compareTo("PrevLvl") == 0) {
					prevLvl();
					}
				else if (com.compareTo("FwdLvl") == 0) {
					fwdLvl();
					}
				}
			});
		editFrame.setManipulationListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String com = ae.getActionCommand();
					 if (com.compareTo("View") == 0) {
					 	showViewer();
					 	viewFrame.displaySpell(currentSpell);
						}
				else if (com.compareTo("Save") == 0) {
					Arcana.SpellRecord spellRec = new Arcana.SpellRecord();
					spellRec = editFrame.getSpell();
					// need to check for empty record (it should not be added)
					spellBook.addSpellRecord(spellRec);
					touch();
					lastSpell = spellRec;
					currentSpell = spellRec;
					
					// !!! Check for duplicate spells by checking spell name (no two spells can have same name)
					// the above is now taken care of in SpellBook addSpellRecord
					
					if (spellBook.getSpellRecordVector(spellRec.getLevel()).contains(spellRec)) {
						hostFrame.printEndLine("Spell Added to Book...");
						}
					else hostFrame.printEndLine("Spell Not Added!  FUCK! A BUG!");
					
					//editFrame.setVisible(false);
					editFrame.clearFields();
					viewFrame.displaySpell(currentSpell);
					showViewer();
					}
				else if (com.compareTo("Clear") == 0) {
					editFrame.clearFields();
					}
				else if (com.compareTo("New") == 0) {
					editFrame.clearFields();
					}
				}
			});
		}
		
	private void initViewer() {
		viewFrame.setNavigationListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String com = ae.getActionCommand();
					 if (com.compareTo("Back") == 0) {
						backSpell();
						}
				else if (com.compareTo("Next") == 0) {
					nextSpell();
					}
				else if (com.compareTo("PrevLvl") == 0) {
					prevLvl();
					}
					
				else if (com.compareTo("FwdLvl") == 0) {
					fwdLvl();
					}
				}
			});
		viewFrame.setManipulationListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String com = ae.getActionCommand();
					 if (com.compareTo("Edit") == 0) {
					 	editSpell();
						}
				else if (com.compareTo("Delete") == 0) {
					// need to implement after delete function is implemented
					deleteSpell();
					}
				else if (com.compareTo("Find") == 0) {
					// need to implement w/ JOptionPane
					find();
					}
				else if (com.compareTo("New") == 0) {
					newSpell();
					}
				}
			});
		}
		
	private void initBrowser() {
		browserFrame.setBackListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (browserFrame.getLevelList().hasFocus()) {
					prevLvl();
					}
				if (browserFrame.getSpellsList().hasFocus()) {
					backSpell();
					}
				}
			});
		browserFrame.setFwdListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (browserFrame.getLevelList().hasFocus()) {
					fwdLvl();
					}
				if (browserFrame.getSpellsList().hasFocus()) {
					nextSpell();
					}
				}
			});
		browserFrame.setViewListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				viewSpell();
				}
			});
		browserFrame.setEditListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				editSpell();
				}
			});
		browserFrame.setDeleteListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				deleteSpell();
				}
			});
		browserFrame.setLoadListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				//deleteSpell();
				}
			});
		browserFrame.setSaveListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				//deleteSpell();
				}
			});
		browserFrame.setSaveAsListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				//deleteSpell();
				}
			});
		browserFrame.setLevelListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent lse) {
				currentLevel = ((JList)lse.getSource()).getSelectedIndex() + 1;
				browserFrame.changeSpellsModel(spellBook.getSpellRegisterVector(currentLevel)
													   .getSpellNamesVector());
				}
			});
		browserFrame.setSpellsListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent lse) {
				currentIndexInLevel = ((JList)lse.getSource()).getSelectedIndex();
				if (currentIndexInLevel < 0) { currentIndexInLevel = 0; }
				currentSpell = spellBook.getSpellRegisterVector(currentLevel)
										.getSpellRegister(currentIndexInLevel)
										.getSourceSpell();
				}
			});
		browserFrame.setSpellsListMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				if (me.getID() == MouseEvent.MOUSE_CLICKED && me.getClickCount() >= 2) {
					viewSpell();
					}
				}
			});
		}
		
	private void showEditor() {
		viewFrame.setVisible(false);
		editFrame.displaySpell(currentSpell);
		editFrame.setVisible(true);
		}
		
	private void showViewer() {
		editFrame.setVisible(false);
		viewFrame.displaySpell(currentSpell);
		viewFrame.setVisible(true);
		}
		
	private void showBrowser() {
		browserFrame.changeSpellsModel(spellBook.getSpellRegisterVector(currentLevel)
												.getSpellNamesVector());
		browserFrame.setVisible(true);
		}
		
	private void touch() {
		browserFrame.undimSave();
		fileTouched = true; 
		}
	
	private void menuSave() {
		if (fileTouched) {
			spellFile.save(spellBook);
			browserFrame.dimSave();
			fileTouched = false;
			}
		}
		
	private void menuLoad() {
		if (fileTouched) {
			int choice = JOptionPane.showConfirmDialog(browserFrame, "This book is not saved! Load anyways? (i.e. data loss)", 
							"Book Not Saved!", JOptionPane.OK_CANCEL_OPTION, 
							JOptionPane.WARNING_MESSAGE);
			switch (choice) {
				case JOptionPane.OK_OPTION:
					spellBook = spellFile.loadFrom((Component)browserFrame);  // what the fuck is this?  trying something else...
					//spellBook = spellFile.load();
					currentSpell = spellBook.getSpellRegisterVector(currentLevel)
											.getSpellRegister(currentIndexInLevel)
											.getSourceSpell();
					break;
				case JOptionPane.CANCEL_OPTION:
				case JOptionPane.CLOSED_OPTION:
					break;
				}
			}
		}
		
	private void menuSaveAs() {
		spellFile.saveAs(spellBook, (Component)browserFrame);
		}
	
	private void prevLvl() {
		if (currentLevel - 1 >= 1) {
			--currentLevel;
			currentIndexInLevel = 0;
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(currentIndexInLevel)
									.getSourceSpell();
			browserFrame.changeSpellsModel(spellBook.getSpellRegisterVector(currentLevel)
													.getSpellNamesVector());
			browserFrame.getSpellsList().setSelectedIndex(currentIndexInLevel);
			browserFrame.getLevelList().setSelectedIndex(currentLevel - 1); // to be an index
			viewFrame.displaySpell(currentSpell);
			editFrame.displaySpell(currentSpell);
			}
		}
		
	private void fwdLvl() {
		if (currentLevel + 1 <= 9) {
			++currentLevel;
			currentIndexInLevel = 0;
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
									.getSpellRegister(currentIndexInLevel)
									.getSourceSpell();
			browserFrame.changeSpellsModel(spellBook.getSpellRegisterVector(currentLevel)
													.getSpellNamesVector());
			browserFrame.getSpellsList().setSelectedIndex(currentIndexInLevel);
			browserFrame.getLevelList().setSelectedIndex(currentLevel - 1); // to be an index
			viewFrame.displaySpell(currentSpell);
			editFrame.displaySpell(currentSpell);
			}
		}
		
	private void nextSpell() {
		if (!(currentIndexInLevel >= spellBook.getSpellRegisterVector(currentLevel).getSpellRegisterVector().size() - 1)) {
			++currentIndexInLevel;
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
			browserFrame.getSpellsList().setSelectedIndex(currentIndexInLevel);
			viewFrame.displaySpell(currentSpell);
			editFrame.displaySpell(currentSpell);
			}
		}
	
	private void backSpell() {
		if (!(currentIndexInLevel <= 0)) {
			--currentIndexInLevel;
			currentSpell = spellBook.getSpellRegisterVector(currentLevel)
													.getSpellRegister(currentIndexInLevel)
													.getSourceSpell();
			browserFrame.getSpellsList().setSelectedIndex(currentIndexInLevel);
			viewFrame.displaySpell(currentSpell);
			editFrame.displaySpell(currentSpell);
			}
		}
		
	private void newSpell() {
		showEditor();
		editFrame.clearFields();
		}
		
	private void editSpell() { showEditor(); }
		
	private void viewSpell() { showViewer(); }
		
	private void deleteSpell() {
		removeSpell();
		viewFrame.displaySpell(currentSpell);
		}
	
	private void find() {
		String searchStr = JOptionPane.showInputDialog(browserFrame, "Enter the spell's name:", "Find", JOptionPane.QUESTION_MESSAGE);
		if (searchStr != null) {
			currentSpell = spellBook.findSpellByName(searchStr);
			if (currentSpell.getName().compareTo("Untitled") == 0) {
				JOptionPane.showMessageDialog(browserFrame, "That spell was not found!", 
										"Error: Spell Not Found", JOptionPane.ERROR_MESSAGE);
				}
			else showViewer();
			}
		}

	// Main entry point
	static public void main(String[] args) {
		SpellCompositor spellCompApp = new SpellCompositor();
		spellCompApp.mainline();
		}
	
}
