/*
	SpellRecord
		- Data-structure for the storage of spell information
	
	Written by Scott C. Ziegler for use with Simulcra RPG and AD&D.
	
	This file and its intellectual contents are considered property of the creator and are to be treated as such with respect to use in other applications.  Any use of this software for any purpose whatsoever must have explicit permission from the author of the file.  This code is part of an ongoing development process for future possible products, and so is considered private property.
Do not use without permission.  Author: Scott C. Ziegler <Aslan@Narnia.net>
	
	vers. history:
		05.31.00 - added the School property to SpellRecord (it was erroneously
					absent.)
		07.08.00 - fixed syntactic bug replacing append with setText.
		07.08.00 - achieved a syntactically correct version.
		08.23.00 - realized a major conceptual error in design and began to refit
		09.07.00 - reached a syntactically correct version.
	
	Work Needed:
		- fix the components by string, as m is in somantic and may trigger material
	
	Work Completed:
		- Fixed append/setText bug 
		- fixed pervasive, conceptual error in DSs.
		- added SpellRegister and SpellRecord members to the DSs.
		


*/
package Arcana;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.awt.Color;


// import SpellBookTest.Sorter;

// SpellRecord
//
//		Data-structure for the storage of spell information

public class SpellRecord implements Sorter.Comparable, Serializable {

	int 	Level;
	String 	Name;
	String 	School;
	String 	Range;
	int 	Components;
	String 	Duration;
	String 	CastingTime;
	String 	AreaOfEffect;
	String 	SavingThrow;
	
	SpellDescription effect;
	
	static final int VERBAL = 1;
	static final int SOMANTIC = 2;
	static final int MATERIAL = 4;
	static final int VERBAL_SOMANTIC = 3;
	static final int SOMANTIC_MATERIAL = 6;
	static final int VERBAL_MATERIAL = 5;
	static final int VERBAL_SOMANTIC_MATERIAL = 7;
	
	SpellRegister	register; // need to implement this though class
	
	public SpellRecord(int lvl, String name, String school, String range, 
					int components, String duration, String castTime, String aoe, String save) {
				
		Level = lvl;
		Name = name;
		School = school;
		Range = range;
		Components = components;
		Duration = duration;
		CastingTime = castTime;
		AreaOfEffect = aoe;
		SavingThrow = save;
		
		register = new SpellRegister(this);
		effect = new SpellDescription("Bang!\n");
		}
		
	public SpellRecord() {
		Level = 1;
		Name = "Untitled";
		School = "Abjuration";
		Range = "0";
		Components = VERBAL_SOMANTIC_MATERIAL;
		Duration = "1 Round";
		CastingTime = "1";
		AreaOfEffect = "Caster";
		SavingThrow = "Negates";
		
		register = new SpellRegister(this);
		effect = new SpellDescription("Bang!\n");
		}
	
	// Mutators
	
	public void setLevel(int lvl) {
		Level = lvl;
		register.setLevel(lvl);
		}
		
	public void setName(String name) {
		Name = name;
		register.setName(name);
		}
	
	public void setSchool(String school) {
		School = school;
		register.setSchool(school);
		}
	
	public void setRange(String range) {
		Range = range;
		}
		
	public void setComponents(int components) {
		Components = components;
		register.setComponents(components);
		}
		
	public void setComponentsByString(String str) {
		Components = 0;
		if (str.indexOf("V") != -1) {
			Components += VERBAL;
			}
		if (str.indexOf("S") != -1) {
			Components += SOMANTIC;
			}
		if (str.indexOf("M") != -1) { // Oops?! soMantic
			Components += MATERIAL;
			}
		}
	
	public void setDuration(String duration) {
		Duration = duration;
		}
		
	public void setCastingTime(String castingTime) {
		CastingTime = castingTime;
		}
		
	public void setAreaOfEffect(String areaOfEffect) {
		AreaOfEffect = areaOfEffect;
		}
		
	public void setSavingThrow(String savingThrow) {
		SavingThrow = savingThrow;
		}
		
	public void setEffect(SpellDescription desc) {
		effect = desc;
		}
	
	public void setSpellRegister(SpellRegister spReg) {
		register = spReg;
		}
	
	
	
	// Selectors
	
	public int getLevel() {
		return Level;
		}
	
	public String getName() {
		return Name;
		}
	
	public String getSchool() {
		return School;
		}
	
	public String getRange() {
		return Range;
		}
		
	public int getComponents() {
		return Components;
		}
		
	public String getComponentsString() {
		switch(Components) {
			case VERBAL :
				return new String("V");
			case SOMANTIC :
				return new String("S");
			case MATERIAL :
				return new String("M");
			case VERBAL_SOMANTIC :
				return new String("V,S");
			case SOMANTIC_MATERIAL:
				return new String("S,M");
			case VERBAL_MATERIAL :
				return new String("V,M");
			case VERBAL_SOMANTIC_MATERIAL :
				return new String("V,S,M");
			default :
				return new String("V");
			}
		}
		
	public String getDuration() {
		return Duration;
		}
		
	public String getCastingTime() {
		return CastingTime;
		}
		
	public String getAreaOfEffect() {
		return AreaOfEffect;
		}
		
	public String getSavingThrow() {
		return SavingThrow;
		}
		
	public SpellDescription getEffect() {
		return effect;
		}
		
	public SpellRegister getSpellRegister() {
		return register;
		}
		
	// Interface - Comparable
	
	public int compareTo(Object other) {
		return Name.compareTo(((SpellRecord)other).getName());
		}
	
	}
	
	

	


	
	

	
	
